"""
Add glider to configuration file.

    Usage:

        add-glider glider

    or

        python add_glider.py path/to/config/file glider
"""

import os
import sys
import errno
import lockfile as lf
import ConfigParser as cp

def _main():
    """Add a glider section to the configuration file.

    These doctests require write permissions to tests/etc.

    Too few arguments:

    >>> import shutil
    >>> cfg = os.path.join(os.path.dirname(__file__),
    ...                    "tests", "etc", "autopilot.cfg")
    >>> shutil.copy2(cfg + ".test", cfg)    
    >>> sys.argv = [__file__,
    ...             cfg,]
    >>> _main()
    <BLANKLINE>
    Add glider to configuration file.
    <BLANKLINE>
        Usage:
    <BLANKLINE>
            add-glider glider
    <BLANKLINE>
        or
    <BLANKLINE>
            python add_glider.py path/to/config/file glider
    <BLANKLINE>

    Too many arguments:

    >>> import shutil
    >>> cfg = os.path.join(os.path.dirname(__file__),
    ...                    "tests", "etc", "autopilot.cfg")
    >>> shutil.copy2(cfg + ".test", cfg)    
    >>> sys.argv = [__file__,
    ...             cfg,
    ...             "Cthulhu",
    ...             "Kraken",]
    >>> _main()
    <BLANKLINE>
    Add glider to configuration file.
    <BLANKLINE>
        Usage:
    <BLANKLINE>
            add-glider glider
    <BLANKLINE>
        or
    <BLANKLINE>
            python add_glider.py path/to/config/file glider
    <BLANKLINE>

    Missing configuration file:

    >>> from StringIO import StringIO
    >>> cfg = os.path.join(os.path.dirname(__file__),
    ...                    "tests", "etc", "autopilot.cfg")
    >>> try:
    ...     os.remove(cfg)
    ... except:
    ...     pass
    >>> sys.argv = [__file__,
    ...             cfg,
    ...             "Cthulhu",]
    >>> output = StringIO()
    >>> save = sys.stdout
    >>> sys.stdout = output
    >>> _main()
    >>> sys.stdout = save
    >>> output.getvalue() == "Glider cthulhu added to {0}.\\n". \\
    ...                      format(os.path.abspath(cfg))
    True
    >>> output.close()
    >>>
    >>> import autopilot.list_gliders
    >>> sys.argv = [__file__,
    ...             cfg,]
    >>> autopilot.list_gliders._main()
    cthulhu

    Empty configuration file:

    >>> from StringIO import StringIO
    >>> cfg = os.path.join(os.path.dirname(__file__),
    ...                    "tests", "etc", "autopilot.cfg")
    >>> try:
    ...     os.remove(cfg)
    ...     open(cfg, 'w').close()
    ... except:
    ...     pass
    >>> sys.argv = [__file__,
    ...             cfg,
    ...             "Cthulhu",]
    >>> output = StringIO()
    >>> save = sys.stdout
    >>> sys.stdout = output
    >>> _main()
    >>> sys.stdout = save
    >>> output.getvalue() == "Glider cthulhu added to {0}.\\n". \\
    ...                      format(os.path.abspath(cfg))
    True
    >>> output.close()
    >>>
    >>> import autopilot.list_gliders
    >>> sys.argv = [__file__,
    ...             cfg,]
    >>> autopilot.list_gliders._main()
    cthulhu

    Configuration already contains glider:

    >>> import shutil
    >>> from StringIO import StringIO
    >>> cfg = os.path.join(os.path.dirname(__file__),
    ...                    "tests", "etc", "autopilot.cfg")
    >>> shutil.copy2(cfg + ".test", cfg)    
    >>> sys.argv = [__file__,
    ...             cfg,
    ...             "Pelagia"]
    >>> output = StringIO()
    >>> save = sys.stdout
    >>> sys.stdout = output
    >>> _main()
    >>> sys.stdout = save
    >>> output.getvalue() == "Glider pelagia already exists in {0}.\\n". \\
    ...                      format(os.path.abspath(cfg))
    True
    >>> output.close()
    >>>
    >>> import autopilot.list_gliders
    >>> sys.argv = [__file__,
    ...             cfg,]
    >>> autopilot.list_gliders._main() # doctest: +ELLIPSIS
    ramses
    pelagia
    salacia
    sandgnat
    ...

    Regular configuration file:

    >>> import shutil
    >>> from StringIO import StringIO
    >>> cfg = os.path.join(os.path.dirname(__file__),
    ...                    "tests", "etc", "autopilot.cfg")
    >>> shutil.copy2(cfg + ".test", cfg)    
    >>> sys.argv = [__file__,
    ...             cfg,
    ...             "Cthulhu",]
    >>> output = StringIO()
    >>> save = sys.stdout
    >>> sys.stdout = output
    >>> _main()
    >>> sys.stdout = save
    >>> output.getvalue() == "Glider cthulhu added to {0}.\\n". \\
    ...                      format(os.path.abspath(cfg))
    True
    >>> output.close()
    >>>
    >>> import autopilot.list_gliders
    >>> sys.argv = [__file__,
    ...             cfg,]
    >>> autopilot.list_gliders._main() # doctest: +ELLIPSIS
    ramses
    pelagia
    salacia
    sandgnat
    ...
    cthulhu

    """

    num_args = len(sys.argv) - 1
    if num_args != 2:
        print __doc__
    else:
        cfg_path = os.path.abspath(sys.argv[1])
        glider = sys.argv[2].lower()
        report = {"glider": glider,
                  "path": cfg_path,
                 }
        try:
            lock = lf.FileLock(cfg_path)
            lock.acquire(5)
        except lf.LockTimeout:
            lock.break_lock()
            lock.acquire(0)
        config = cp.SafeConfigParser()
        try:
            with open(cfg_path) as handle:
                config.readfp(handle)
        except IOError as e:
            if e.errno != errno.ENOENT:
                lock.release()
                raise
        try:
            config.add_section(glider)
            with open(cfg_path, "w") as handle:
                config.write(handle)
            print "Glider {glider} added to {path}.".format(**report)
        except cp.DuplicateSectionError:
            print "Glider {glider} already exists in {path}.".format(**report)
        finally:
            lock.release()

if __name__ == "__main__":
    _main()
