"""
Add task to glider configuration.

    Usage:

        add-task glider task

    or

        python add_task.py path/to/config/file glider task
"""

import os
import sys
import errno
import lockfile as lf
import ConfigParser as cp

valid_tasks = {"alert":
                  {"description":
                       "Print a surface message on the autopilot console.",
                   "priority": 0,
                  },
               "bearing":
                  {"description":
                       "Generate a goto list based on a bearing.",
                   "priority": 1,
                  },
               "target":
                  {"description":
                       "Generate a goto list based on a target waypoint.",
                   "priority": 2,
                  },
               "track":
                  {"description":
                       "Email a Nobletec file to a subscriber list.",
                   "priority": 3,
                  },
              }

def _usage(valid_only=False):
    """Print usage instructions.

    >>> _usage()
    <BLANKLINE>
    Add task to glider configuration.
    <BLANKLINE>
        Usage:
    <BLANKLINE>
            add-task glider task
    <BLANKLINE>
        or
    <BLANKLINE>
            python add_task.py path/to/config/file glider task
    <BLANKLINE>
        Valid tasks (in order of priority of service):
    <BLANKLINE>
            alert: Print a surface message on the autopilot console.
            bearing: Generate a goto list based on a bearing.
            target: Generate a goto list based on a target waypoint.
            track: Email a Nobletec file to a subscriber list.
    <BLANKLINE>

    >>> _usage(True)
        Valid tasks (in order of priority of service):
    <BLANKLINE>
            alert: Print a surface message on the autopilot console.
            bearing: Generate a goto list based on a bearing.
            target: Generate a goto list based on a target waypoint.
            track: Email a Nobletec file to a subscriber list.
    <BLANKLINE>
    """
    tasks = valid_tasks.items()
    tasks.sort(cmp=lambda x, y: cmp(x[1]["priority"],
                                    y[1]["priority"]))
    if not valid_only:
        print __doc__
    print "    Valid tasks (in order of priority of service):"
    print "\n       ",
    print "\n        ".join(["%s: %s" % (k, v["description"])
                             for k, v in tasks])
    print
    return

def _main():
    """Add task to glider section in configuration file.

    These doctests require write permissions to tests/etc.

    Too few arguments:

    >>> import shutil
    >>> cfg = os.path.join(os.path.dirname(__file__),
    ...                    "tests", "etc", "autopilot.cfg")
    >>> shutil.copy2(cfg + ".test", cfg)    
    >>> sys.argv = [__file__,
    ...             cfg,
    ...             "Cthulhu",]
    >>> _main()
    <BLANKLINE>
    Add task to glider configuration.
    <BLANKLINE>
        Usage:
    <BLANKLINE>
            add-task glider task
    <BLANKLINE>
        or
    <BLANKLINE>
            python add_task.py path/to/config/file glider task
    <BLANKLINE>
        Valid tasks (in order of priority of service):
    <BLANKLINE>
            alert: Print a surface message on the autopilot console.
            bearing: Generate a goto list based on a bearing.
            target: Generate a goto list based on a target waypoint.
            track: Email a Nobletec file to a subscriber list.
    <BLANKLINE>

    Too many arguments:

    >>> import shutil
    >>> cfg = os.path.join(os.path.dirname(__file__),
    ...                    "tests", "etc", "autopilot.cfg")
    >>> shutil.copy2(cfg + ".test", cfg)    
    >>> sys.argv = [__file__,
    ...             cfg,
    ...             "Cthulhu",
    ...             "alert",
    ...             "O'R'lyeh",]
    >>> _main()
    <BLANKLINE>
    Add task to glider configuration.
    <BLANKLINE>
        Usage:
    <BLANKLINE>
            add-task glider task
    <BLANKLINE>
        or
    <BLANKLINE>
            python add_task.py path/to/config/file glider task
    <BLANKLINE>
        Valid tasks (in order of priority of service):
    <BLANKLINE>
            alert: Print a surface message on the autopilot console.
            bearing: Generate a goto list based on a bearing.
            target: Generate a goto list based on a target waypoint.
            track: Email a Nobletec file to a subscriber list.
    <BLANKLINE>

    Missing configuration file:

    >>> from StringIO import StringIO
    >>> cfg = os.path.join(os.path.dirname(__file__),
    ...                    "tests", "etc", "autopilot.cfg")
    >>> try:
    ...     os.remove(cfg)
    ... except:
    ...     pass
    >>> sys.argv = [__file__,
    ...             cfg,
    ...             "Cthulhu",
    ...             "alert",]
    >>> output = StringIO()
    >>> save = sys.stdout
    >>> sys.stdout = output
    >>> _main()
    >>> sys.stdout = save
    >>> output.getvalue() == "Glider cthulhu does not exist in {0}.\\n". \\
    ...                      format(os.path.abspath(cfg))
    True
    >>> output.close()

    Empty configuration file:

    >>> from StringIO import StringIO
    >>> cfg = os.path.join(os.path.dirname(__file__),
    ...                    "tests", "etc", "autopilot.cfg")
    >>> try:
    ...     os.remove(cfg)
    ...     open(cfg, 'w').close()
    ... except:
    ...     pass
    >>> sys.argv = [__file__,
    ...             cfg,
    ...             "Cthulhu",
    ...             "alert",]
    >>> output = StringIO()
    >>> save = sys.stdout
    >>> sys.stdout = output
    >>> _main()
    >>> sys.stdout = save
    >>> output.getvalue() == "Glider cthulhu does not exist in {0}.\\n". \\
    ...                      format(os.path.abspath(cfg))
    True
    >>> output.close()

    Invalid task:

    >>> import shutil
    >>> cfg = os.path.join(os.path.dirname(__file__),
    ...                    "tests", "etc", "autopilot.cfg")
    >>> shutil.copy2(cfg + ".test", cfg)    
    >>> sys.argv = [__file__,
    ...             cfg,
    ...             "Cthulhu",
    ...             "summon",]
    >>> _main()
    <BLANKLINE>
    Invalid task: summon
    <BLANKLINE>
        Valid tasks (in order of priority of service):
    <BLANKLINE>
            alert: Print a surface message on the autopilot console.
            bearing: Generate a goto list based on a bearing.
            target: Generate a goto list based on a target waypoint.
            track: Email a Nobletec file to a subscriber list.
    <BLANKLINE>

    Configuration does not contain glider:

    >>> import shutil
    >>> from StringIO import StringIO
    >>> cfg = os.path.join(os.path.dirname(__file__),
    ...                    "tests", "etc", "autopilot.cfg")
    >>> shutil.copy2(cfg + ".test", cfg)    
    >>> sys.argv = [__file__,
    ...             cfg,
    ...             "Cthulhu",
    ...             "Alert",]
    >>> output = StringIO()
    >>> save = sys.stdout
    >>> sys.stdout = output
    >>> _main()
    >>> sys.stdout = save
    >>> output.getvalue() == "Glider cthulhu added to {0}.\\n". \\
    ...                      format(os.path.abspath(cfg)) + \\
    ...                      "The alert task for glider cthulhu " + \\
    ...                      "added to {0}.\\n". \\
    ...                      format(os.path.abspath(cfg))
    True
    >>> output.close()
    >>>
    >>> import autopilot.list_gliders
    >>> sys.argv = [__file__,
    ...             cfg,]
    >>> autopilot.list_gliders._main() # doctest: +ELLIPSIS
    ramses
    pelagia
    salacia
    sandgnat
    ...
    cthulhu
    >>>
    >>> import autopilot.list_tasks
    >>> sys.argv = [__file__,
    ...             cfg,
    ...             "Cthulhu",]
    >>> autopilot.list_tasks._main()
    alert

    Glider does not contain task list:

    >>> import shutil
    >>> from StringIO import StringIO
    >>> cfg = os.path.join(os.path.dirname(__file__),
    ...                    "tests", "etc", "autopilot.cfg")
    >>> shutil.copy2(cfg + ".test", cfg)    
    >>> sys.argv = [__file__,
    ...             cfg,
    ...             "Ramses",
    ...             "Alert",]
    >>> output = StringIO()
    >>> save = sys.stdout
    >>> sys.stdout = output
    >>> _main()
    >>> sys.stdout = save
    >>> output.getvalue() == "The alert task for glider ramses " + \\
    ...                      "added to {0}.\\n". \\
    ...                      format(os.path.abspath(cfg))
    True
    >>> output.close()
    >>>
    >>> import autopilot.list_tasks
    >>> sys.argv = [__file__,
    ...             cfg,
    ...             "Ramses",]
    >>> autopilot.list_tasks._main()
    alert

    Task list for glider is empty:

    >>> import shutil
    >>> from StringIO import StringIO
    >>> cfg = os.path.join(os.path.dirname(__file__),
    ...                    "tests", "etc", "autopilot.cfg")
    >>> shutil.copy2(cfg + ".test", cfg)    
    >>> sys.argv = [__file__,
    ...             cfg,
    ...             "Pelagia",
    ...             "Alert",]
    >>> output = StringIO()
    >>> save = sys.stdout
    >>> sys.stdout = output
    >>> _main()
    >>> sys.stdout = save
    >>> output.getvalue() == "The alert task for glider pelagia " + \\
    ...                      "added to {0}.\\n". \\
    ...                      format(os.path.abspath(cfg))
    True
    >>> output.close()
    >>>
    >>> import autopilot.list_tasks
    >>> sys.argv = [__file__,
    ...             cfg,
    ...             "Pelagia",]
    >>> autopilot.list_tasks._main()
    alert

    Regular configuration file:

    >>> import shutil
    >>> from StringIO import StringIO
    >>> cfg = os.path.join(os.path.dirname(__file__),
    ...                    "tests", "etc", "autopilot.cfg")
    >>> shutil.copy2(cfg + ".test", cfg)    
    >>> sys.argv = [__file__,
    ...             cfg,
    ...             "Salacia",
    ...             "Alert",]
    >>> output = StringIO()
    >>> save = sys.stdout
    >>> sys.stdout = output
    >>> _main()
    >>> sys.stdout = save
    >>> output.getvalue() == "The alert task for glider salacia " + \\
    ...                      "added to {0}.\\n". \\
    ...                      format(os.path.abspath(cfg))
    True
    >>> output.close()
    >>>
    >>> import autopilot.list_tasks
    >>> sys.argv = [__file__,
    ...             cfg,
    ...             "Salacia",]
    >>> autopilot.list_tasks._main()
    alert
    bearing
    track

    Duplicate task:

    >>> import shutil
    >>> from StringIO import StringIO
    >>> cfg = os.path.join(os.path.dirname(__file__),
    ...                    "tests", "etc", "autopilot.cfg")
    >>> shutil.copy2(cfg + ".test", cfg)    
    >>> sys.argv = [__file__,
    ...             cfg,
    ...             "Salacia",
    ...             "Bearing",]
    >>> output = StringIO()
    >>> save = sys.stdout
    >>> sys.stdout = output
    >>> _main()
    >>> sys.stdout = save
    >>> output.getvalue() == "The bearing task for glider salacia " + \\
    ...                      "already exists in {0}.\\n". \\
    ...                      format(os.path.abspath(cfg))
    True
    >>> output.close()

    Replace bearing task with target task:

    >>> import shutil
    >>> from StringIO import StringIO
    >>> cfg = os.path.join(os.path.dirname(__file__),
    ...                    "tests", "etc", "autopilot.cfg")
    >>> shutil.copy2(cfg + ".test", cfg)    
    >>> sys.argv = [__file__,
    ...             cfg,
    ...             "Salacia",
    ...             "Target",]
    >>> output = StringIO()
    >>> save = sys.stdout
    >>> sys.stdout = output
    >>> _main()
    >>> sys.stdout = save
    >>> output.getvalue() == "The bearing task for glider salacia " + \\
    ...                      "deleted from {0}.\\n". \\
    ...                      format(os.path.abspath(cfg)) + \\
    ...                      "The target task for glider salacia " + \\
    ...                      "added to {0}.\\n". \\
    ...                      format(os.path.abspath(cfg))
    True
    >>> output.close()
    >>>
    >>> import autopilot.list_tasks
    >>> sys.argv = [__file__,
    ...             cfg,
    ...             "Salacia",]
    >>> autopilot.list_tasks._main()
    target
    track

    Replace target task with bearing task:

    >>> import shutil
    >>> from StringIO import StringIO
    >>> cfg = os.path.join(os.path.dirname(__file__),
    ...                    "tests", "etc", "autopilot.cfg")
    >>> shutil.copy2(cfg + ".test", cfg)    
    >>> sys.argv = [__file__,
    ...             cfg,
    ...             "Sandgnat",
    ...             "Bearing",]
    >>> output = StringIO()
    >>> save = sys.stdout
    >>> sys.stdout = output
    >>> _main()
    >>> sys.stdout = save
    >>> output.getvalue() == "The target task for glider sandgnat " + \\
    ...                      "deleted from {0}.\\n". \\
    ...                      format(os.path.abspath(cfg)) + \\
    ...                      "The bearing task for glider sandgnat " + \\
    ...                      "added to {0}.\\n". \\
    ...                      format(os.path.abspath(cfg))
    True
    >>> output.close()
    >>>
    >>> import autopilot.list_tasks
    >>> sys.argv = [__file__,
    ...             cfg,
    ...             "Sandgnat",]
    >>> autopilot.list_tasks._main()
    alert
    bearing
    track

    """

    num_args = len(sys.argv) - 1
    if num_args != 3:
        _usage()
    else:
        cfg_path = os.path.abspath(sys.argv[1])
        glider = sys.argv[2].lower()
        task = sys.argv[3].lower()
        if task not in valid_tasks:
            print "\nInvalid task: {0}\n".format(task)
            _usage(True)
        else:
            report = {"glider": glider,
                      "path": cfg_path,
                      "task": task,
                     }
            try:
                lock = lf.FileLock(cfg_path)
                lock.acquire(5)
            except lf.LockTimeout:
                lock.break_lock()
                lock.acquire(0)
            config = cp.SafeConfigParser()
            try:
                with open(cfg_path) as handle:
                    config.readfp(handle)
            except IOError as e:
                if e.errno == errno.ENOENT:
                    print "Glider {glider} does not exist in {path}.". \
                          format(**report)
                else:
                    raise
            else:
                try:
                    tasks = config.get(glider, "tasks").split()
                    if "bearing" in tasks and task == "target":
                        tasks.remove("bearing")
                        print "The bearing task for glider {glider} " \
                              "deleted from {path}.". \
                              format(**report)
                    elif "target" in tasks and task == "bearing":
                        tasks.remove("target")
                        print "The target task for glider {glider} " \
                              "deleted from {path}.". \
                              format(**report)
                except cp.NoOptionError:
                    tasks = []
                except cp.NoSectionError:
                    config.add_section(glider)
                    print "Glider {glider} added to {path}.".format(**report)
                    tasks = []
                if task in tasks:
                    print "The {task} task for glider {glider} " \
                          "already exists in {path}.". \
                          format(**report)
                else:
                    tasks.append(task)
                    tasks.sort(cmp=lambda x, y:
                                       cmp(valid_tasks[x]["priority"],
                                           valid_tasks[y]["priority"]))
                    config.set(glider, "tasks", " ".join(tasks))
                    with open(cfg_path, "w") as handle:
                        config.write(handle)
                    print "The {task} task for glider {glider} " \
                          "added to {path}.". \
                          format(**report)
            finally:
                lock.release()


if __name__ == "__main__":
    _main()
