"""
Delete glider from configuration file.

    Usage:

        delete-glider glider

    or

        python delete_glider.py path/to/config/file glider
"""

import os
import sys
import errno
import lockfile as lf
import ConfigParser as cp

def _main():
    """Delete a glider section from the configuration file.

    These doctests require write permissions to tests/etc.

    Too few arguments:

    >>> import shutil
    >>> cfg = os.path.join(os.path.dirname(__file__),
    ...                    "tests", "etc", "autopilot.cfg")
    >>> shutil.copy2(cfg + ".test", cfg)    
    >>> sys.argv = [__file__,
    ...             cfg,]
    >>> _main()
    <BLANKLINE>
    Delete glider from configuration file.
    <BLANKLINE>
        Usage:
    <BLANKLINE>
            delete-glider glider
    <BLANKLINE>
        or
    <BLANKLINE>
            python delete_glider.py path/to/config/file glider
    <BLANKLINE>

    Too many arguments:

    >>> import shutil
    >>> cfg = os.path.join(os.path.dirname(__file__),
    ...                    "tests", "etc", "autopilot.cfg")
    >>> shutil.copy2(cfg + ".test", cfg)    
    >>> sys.argv = [__file__,
    ...             cfg,
    ...             "Cthulhu",
    ...             "Kraken",]
    >>> _main()
    <BLANKLINE>
    Delete glider from configuration file.
    <BLANKLINE>
        Usage:
    <BLANKLINE>
            delete-glider glider
    <BLANKLINE>
        or
    <BLANKLINE>
            python delete_glider.py path/to/config/file glider
    <BLANKLINE>

    Missing configuration file:

    >>> from StringIO import StringIO
    >>> cfg = os.path.join(os.path.dirname(__file__),
    ...                    "tests", "etc", "autopilot.cfg")
    >>> try:
    ...     os.remove(cfg)
    ... except:
    ...     pass
    >>> sys.argv = [__file__,
    ...             cfg,
    ...             "Cthulhu",]
    >>> output = StringIO()
    >>> save = sys.stdout
    >>> sys.stdout = output
    >>> _main()
    >>> sys.stdout = save
    >>> output.getvalue() == "Glider cthulhu does not exist in {0}.\\n". \\
    ...                      format(os.path.abspath(cfg))
    True
    >>> output.close()

    Empty configuration file:

    >>> from StringIO import StringIO
    >>> cfg = os.path.join(os.path.dirname(__file__),
    ...                    "tests", "etc", "autopilot.cfg")
    >>> try:
    ...     os.remove(cfg)
    ...     open(cfg, 'w').close()
    ... except:
    ...     pass
    >>> sys.argv = [__file__,
    ...             cfg,
    ...             "Cthulhu",]
    >>> output = StringIO()
    >>> save = sys.stdout
    >>> sys.stdout = output
    >>> _main()
    >>> sys.stdout = save
    >>> output.getvalue() == "Glider cthulhu does not exist in {0}.\\n". \\
    ...                      format(os.path.abspath(cfg))
    True
    >>> output.close()

    Configuration does not contain glider:

    >>> import shutil
    >>> from StringIO import StringIO
    >>> cfg = os.path.join(os.path.dirname(__file__),
    ...                    "tests", "etc", "autopilot.cfg")
    >>> shutil.copy2(cfg + ".test", cfg)    
    >>> sys.argv = [__file__,
    ...             cfg,
    ...             "Cthulhu",]
    >>> output = StringIO()
    >>> save = sys.stdout
    >>> sys.stdout = output
    >>> _main()
    >>> sys.stdout = save
    >>> output.getvalue() == "Glider cthulhu does not exist in {0}.\\n". \\
    ...                      format(os.path.abspath(cfg))
    True
    >>> output.close()
    >>>
    >>> import autopilot.list_gliders
    >>> sys.argv = [__file__,
    ...             cfg,]
    >>> autopilot.list_gliders._main() # doctest: +ELLIPSIS
    ramses
    pelagia
    salacia
    sandgnat
    ...

    Regular configuration file:

    >>> import shutil
    >>> from StringIO import StringIO
    >>> cfg = os.path.join(os.path.dirname(__file__),
    ...                    "tests", "etc", "autopilot.cfg")
    >>> shutil.copy2(cfg + ".test", cfg)    
    >>> sys.argv = [__file__,
    ...             cfg,
    ...             "Pelagia"]
    >>> output = StringIO()
    >>> save = sys.stdout
    >>> sys.stdout = output
    >>> _main()
    >>> sys.stdout = save
    >>> output.getvalue() == "Glider pelagia deleted from {0}.\\n". \\
    ...                      format(os.path.abspath(cfg))
    True
    >>> output.close()
    >>>
    >>> import autopilot.list_gliders
    >>> sys.argv = [__file__,
    ...             cfg,]
    >>> autopilot.list_gliders._main() # doctest: +ELLIPSIS
    ramses
    salacia
    sandgnat
    ...

    """

    num_args = len(sys.argv) - 1
    if num_args != 2:
        print __doc__
    else:
        cfg_path = os.path.abspath(sys.argv[1])
        glider = sys.argv[2].lower()
        report = {"glider": glider,
                  "path": cfg_path,
                 }
        try:
            lock = lf.FileLock(cfg_path)
            lock.acquire(5)
        except lf.LockTimeout:
            lock.break_lock()
            lock.acquire(0)
        config = cp.SafeConfigParser()
        try:
            with open(cfg_path) as handle:
                config.readfp(handle)
        except IOError as e:
            if e.errno == errno.ENOENT:
                print "Glider {glider} does not exist in {path}.". \
                      format(**report)
        else:
            if config.remove_section(glider):
                with open(cfg_path, "w") as handle:
                    config.write(handle)
                print "Glider {glider} deleted from {path}.". \
                      format(**report)
            else:
                print "Glider {glider} does not exist in {path}.". \
                      format(**report)
        finally:
            lock.release()

if __name__ == "__main__":
    _main()
