"""
List gliders in configuration file.

    Usage:

        list-gliders

    or

        python list_gliders.py path/to/config/file
"""

import os
import sys
import errno
import lockfile as lf
import ConfigParser as cp

def _main():
    """List all the glider sections in the configution file.

    These doctests require write permission to tests/etc.

    Too many arguments:

    >>> import shutil
    >>> cfg = os.path.join(os.path.dirname(__file__),
    ...                    "tests", "etc", "autopilot.cfg")
    >>> shutil.copy2(cfg + ".test", cfg)    
    >>> sys.argv = [__file__,
    ...             cfg,
    ...             "Cthulhu",
    ...             "alert",
    ...             "Kraken",]
    >>> _main()
    <BLANKLINE>
    List gliders in configuration file.
    <BLANKLINE>
        Usage:
    <BLANKLINE>
            list-gliders
    <BLANKLINE>
        or
    <BLANKLINE>
            python list_gliders.py path/to/config/file
    <BLANKLINE>

    Missing configuration file:

    >>> from StringIO import StringIO
    >>> cfg = os.path.join(os.path.dirname(__file__),
    ...                    "tests", "etc", "autopilot.cfg")
    >>> try:
    ...     os.remove(cfg)
    ... except:
    ...     pass
    >>> sys.argv = [__file__,
    ...             cfg,]
    >>> output = StringIO()
    >>> save = sys.stdout
    >>> sys.stdout = output
    >>> _main()
    >>> sys.stdout = save
    >>> output.getvalue() == "There are no gliders in {0}.\\n". \\
    ...                      format(os.path.abspath(cfg))
    True
    >>> output.close()

    No glider section in configuration file:

    >>> from StringIO import StringIO
    >>> cfg = os.path.join(os.path.dirname(__file__),
    ...                    "tests", "etc", "autopilot.cfg")
    >>> try:
    ...     os.remove(cfg)
    ...     open(cfg, 'w').close()
    ... except:
    ...     pass
    >>> sys.argv = [__file__,
    ...             cfg,]
    >>> output = StringIO()
    >>> save = sys.stdout
    >>> sys.stdout = output
    >>> _main()
    >>> sys.stdout = save
    >>> output.getvalue() == "There are no gliders in {0}.\\n". \\
    ...                      format(os.path.abspath(cfg))
    True
    >>> output.close()

    Regular configuration file:

    >>> import shutil
    >>> shutil.copy2(cfg + ".test", cfg)
    >>> sys.argv = [__file__,
    ...             cfg,]
    >>> _main() # doctest: +ELLIPSIS
    ramses
    pelagia
    salacia
    sandgnat
    ...

    """

    num_args = len(sys.argv) - 1
    if num_args != 1:
        print __doc__
    else:
        cfg_path = os.path.abspath(sys.argv[1])
        try:
            lock = lf.FileLock(cfg_path)
            lock.acquire(5)
        except lf.LockTimeout:
            lock.break_lock()
            lock.acquire(0)
        config = cp.SafeConfigParser()
        try:
            with open(cfg_path) as handle:
                config.readfp(handle)
        except IOError as e:
            if e.errno == errno.ENOENT:
                print "There are no gliders in {0}.". \
                      format(cfg_path)
            else:
                raise
        else:
            gliders = config.sections()
            if gliders:
                print "\n".join(gliders)
            else:
                print "There are no gliders at this time in {0}.". \
                      format(cfg_path)
        finally:
            lock.release()

if __name__ == "__main__":
    _main()
